package unikl.disco.tests;

import static org.junit.Assert.assertEquals;

import java.util.LinkedList;
import java.util.List;

import org.junit.BeforeClass;
import org.junit.Test;

import unikl.disco.curves.Curve;
import unikl.disco.misc.Rational;
import unikl.disco.nc.Configuration;
import unikl.disco.nc.TotalFlowAnalysis;
import unikl.disco.nc.SeparateFlowAnalysis;
import unikl.disco.nc.PmooSfaAnalysis;
import unikl.disco.nc.Configuration.Flag;

import unikl.disco.network.Link;
import unikl.disco.network.Flow;
import unikl.disco.network.Network;
import unikl.disco.network.Server;

public class FeedForward_1SC_4Flows_1AC_4Paths
{
	static final Curve service_curve = Curve.createRateLatency( 20, 20 );
	static final Curve arrival_curve = Curve.createTokenBucket( 5, 25 );
	
	static Network network;
	static Server s0, s1, s2, s3;
	static Flow f0, f1, f2, f3;
	static Link l_s0_s1, l_s1_s3, l_s2_s0, l_s0_s3;
	
	@BeforeClass
	static public void createNetwork()
	{
		network = new Network();
		s0 = network.addServer( service_curve );
		s1 = network.addServer( service_curve );
		s2 = network.addServer( service_curve );
		s3 = network.addServer( service_curve );

		try {
			l_s0_s1 = network.addLink( s0, s1 );
			l_s0_s3 = network.addLink( s0, s3 );
			l_s1_s3 = network.addLink( s1, s3 );
			l_s2_s0 = network.addLink( s2, s0 );
			network.addLink( s2, s1 );
			network.addLink( s2, s3 );
		} catch (Exception e) {
			System.out.println( e.toString() );
			return;
		}
		
		List<Link> f0_path = new LinkedList<Link>();
		f0_path.add( l_s0_s1 );
		f0_path.add( l_s1_s3 );
		
		List<Link> f3_path = new LinkedList<Link>();
		f3_path.add( l_s2_s0 );
		f3_path.add( l_s0_s3 );

		try {	
			f0 = network.addFlow( arrival_curve, f0_path );
			f1 = network.addFlow( arrival_curve, s2, s3 );
			f2 = network.addFlow( arrival_curve, s2, s1 );
			f3 = network.addFlow( arrival_curve, f3_path );
		} catch (Exception e) {
			System.out.println( e.toString() );
			return;
		}
	}
	
//--------------------Flow 0--------------------	
	@Test
	public void f0_tfa_fifoMux()
	{
		System.out.println( "Flow of interest : " + f0.toString() );
		System.out.println();
		
		Configuration.setArbMuxFlag( Flag.GLOBALLY_OFF );
		
		System.out.println( "--- Total Flow Analysis (FIFO_MUX) ---" );
		TotalFlowAnalysis tf = new TotalFlowAnalysis( network );
		
		try {
			tf.performEnd2EndAnalysis( f0 );
			System.out.println( "delay bound     : " + tf.getDelayBound() );
			System.out.println( "     per server : " + tf.getServerDelayBoundMap().values().toString() );
			System.out.println( "backlog bound   : " + tf.getBacklogBound() );
			System.out.println( "     per server : " + tf.getServerBacklogBoundMap().values().toString() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "TFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}

		Configuration.setArbMuxFlag( Flag.SERVERLOCAL_FLAG );
		
		assertEquals("TFA FIFO delay", new Rational( 3735, 32 ), tf.getDelayBound() );
		assertEquals("TFA FIFO backlog", new Rational( 975 ), tf.getBacklogBound() );
	}
	
	@Test
	public void f0_tfa_arbMux()
	{
		System.out.println( "Flow of interest : " + f0.toString() );
		System.out.println();
		
		Configuration.setArbMuxFlag( Flag.GLOBALLY_ON );
		
		System.out.println( "--- Total Flow Analysis (ARB_MUX) ---" );
		TotalFlowAnalysis tf = new TotalFlowAnalysis( network );
		
		try {
			tf.performEnd2EndAnalysis( f0 );
			System.out.println( "delay bound     : " + tf.getDelayBound() );
			System.out.println( "     per server : " + tf.getServerDelayBoundMap().values().toString() );
			System.out.println( "backlog bound   : " + tf.getBacklogBound() );
			System.out.println( "     per server : " + tf.getServerBacklogBoundMap().values().toString() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "TFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}

		Configuration.setArbMuxFlag( Flag.SERVERLOCAL_FLAG );

		assertEquals("TFA ARB delay", new Rational( 1370, 3 ), tf.getDelayBound() );
		assertEquals("TFA ARB backlog", new Rational( 1400 ), tf.getBacklogBound() );
	}

	@Test
	public void f0_sfa_fifoMux()
	{
		System.out.println( "Flow of interest : " + f0.toString() );
		System.out.println();

		Configuration.setArbMuxFlag( Flag.GLOBALLY_OFF );
		
		System.out.println( "--- Separated Flow Analysis (FIFO_MUX) ---" );
		SeparateFlowAnalysis sf = new SeparateFlowAnalysis( network );

		try {
			sf.performEnd2EndAnalysis( f0 );
			System.out.println( "e2e SFA SC      : " + sf.getEnd2EndServiceCurve() );
			System.out.println( "     per server : " + sf.getServerLeftOverBetaMap().values().toString() );
			System.out.println( "delay bound     : " + sf.getDelayBound() );
			System.out.println( "backlog bound   : " + sf.getBacklogBound() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "SFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}

		Configuration.setArbMuxFlag( Flag.SERVERLOCAL_FLAG );
		
		assertEquals("SFA FIFO delay", new Rational( 95 ), sf.getDelayBound() );
		assertEquals("SFA FIFO backlog", new Rational( 975, 2 ), sf.getBacklogBound() );
	}

	@Test
	public void f0_sfa_arbMux()
	{
		System.out.println( "Flow of interest : " + f0.toString() );
		System.out.println();
		
		Configuration.setArbMuxFlag( Flag.GLOBALLY_ON );
		
		System.out.println( "--- Separated Flow Analysis (ARB_MUX) ---" );
		SeparateFlowAnalysis sf = new SeparateFlowAnalysis( network );

		try {
			sf.performEnd2EndAnalysis( f0 );
			System.out.println( "e2e SFA SC      : " + sf.getEnd2EndServiceCurve() );
			System.out.println( "     per server : " + sf.getServerLeftOverBetaMap().values().toString() );
			System.out.println( "delay bound     : " + sf.getDelayBound() );
			System.out.println( "backlog bound   : " + sf.getBacklogBound() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "SFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}

		Configuration.setArbMuxFlag( Flag.SERVERLOCAL_FLAG );
		
		assertEquals("SFA ARB delay", new Rational( 1135, 6 ), sf.getDelayBound() );
		assertEquals("SFA ARB backlog", new Rational( 2875, 3 ), sf.getBacklogBound() );
	}
	
	@Test
	public void f0_pmoo_arbMux()
	{
		System.out.println( "Flow of interest : " + f0.toString() );
		System.out.println();
				
		Configuration.setArbMuxFlag( Flag.GLOBALLY_ON );
		
		System.out.println( "--- PMOO Analysis (ARB_MUX) ---" );
		PmooSfaAnalysis pmoo = new PmooSfaAnalysis( network );
		
		try {
			pmoo.performEnd2EndAnalysis( f0 );
			System.out.println( "e2e PMOO SFA SC : " + pmoo.getEnd2EndServiceCurve() );
			System.out.println( "delay bound     : " + pmoo.getDelayBound() );
			System.out.println( "backlog bound   : " + pmoo.getBacklogBound() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "PMOO SFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}

		Configuration.setArbMuxFlag( Flag.SERVERLOCAL_FLAG );

		assertEquals("PMOO ARB delay", new Rational( 875, 4 ), pmoo.getDelayBound() );
		assertEquals("PMOO ARB backlog", new Rational( 4425, 4 ), pmoo.getBacklogBound() );
	}

//--------------------Flow 1--------------------	
	@Test
	public void f1_tfa_fifoMux()
	{
		System.out.println( "Flow of interest : " + f1.toString() );
		System.out.println();
		
		Configuration.setArbMuxFlag( Flag.GLOBALLY_OFF );
		
		System.out.println( "--- Total Flow Analysis (FIFO_MUX) ---" );
		TotalFlowAnalysis tf = new TotalFlowAnalysis( network );
		
		try {
			tf.performEnd2EndAnalysis( f1 );
			System.out.println( "delay bound     : " + tf.getDelayBound() );
			System.out.println( "     per server : " + tf.getServerDelayBoundMap().values().toString() );
			System.out.println( "backlog bound   : " + tf.getBacklogBound() );
			System.out.println( "     per server : " + tf.getServerBacklogBoundMap().values().toString() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "TFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}

		Configuration.setArbMuxFlag( Flag.SERVERLOCAL_FLAG );
		
		assertEquals("TFA FIFO delay", new Rational( 77.5 ), tf.getDelayBound() );
		assertEquals("TFA FIFO backlog", new Rational( 975 ), tf.getBacklogBound() );
	}
	
	@Test
	public void f1_tfa_arbMux()
	{
		System.out.println( "Flow of interest : " + f1.toString() );
		System.out.println();
		
		Configuration.setArbMuxFlag( Flag.GLOBALLY_ON );
		
		System.out.println( "--- Total Flow Analysis (ARB_MUX) ---" );
		TotalFlowAnalysis tf = new TotalFlowAnalysis( network );
		
		try {
			tf.performEnd2EndAnalysis( f1 );
			System.out.println( "delay bound     : " + tf.getDelayBound() );
			System.out.println( "     per server : " + tf.getServerDelayBoundMap().values().toString() );
			System.out.println( "backlog bound   : " + tf.getBacklogBound() );
			System.out.println( "     per server : " + tf.getServerBacklogBoundMap().values().toString() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "TFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}

		Configuration.setArbMuxFlag( Flag.SERVERLOCAL_FLAG );
		
		assertEquals("TFA ARB delay", new Rational( 395 ), tf.getDelayBound() );
		assertEquals("TFA ARB backlog", new Rational( 1400 ), tf.getBacklogBound() );
	}
	
	@Test
	public void f1_sfa_fifoMux()
	{
		System.out.println( "Flow of interest : " + f1.toString() );
		System.out.println();
		
		Configuration.setArbMuxFlag( Flag.GLOBALLY_OFF );
		
		System.out.println( "--- Separated Flow Analysis (FIFO_MUX) ---" );
		SeparateFlowAnalysis sf = new SeparateFlowAnalysis( network );

		try {
			sf.performEnd2EndAnalysis( f1 );
			System.out.println( "e2e SFA SC      : " + sf.getEnd2EndServiceCurve() );
			System.out.println( "     per server : " + sf.getServerLeftOverBetaMap().values().toString() );
			System.out.println( "delay bound     : " + sf.getDelayBound() );
			System.out.println( "backlog bound   : " + sf.getBacklogBound() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "SFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}

		Configuration.setArbMuxFlag( Flag.SERVERLOCAL_FLAG );
		
		assertEquals("SFA FIFO delay", new Rational( 2285, 32 ), sf.getDelayBound() );
		assertEquals("SFA FIFO backlog", new Rational( 11825, 32 ), sf.getBacklogBound() );
	}
	
	@Test
	public void f1_sfa_arbMux()
	{
		System.out.println( "Flow of interest : " + f1.toString() );
		System.out.println();
		
		Configuration.setArbMuxFlag( Flag.GLOBALLY_ON );
		
		System.out.println( "--- Separated Flow Analysis (ARB_MUX) ---" );
		SeparateFlowAnalysis sf = new SeparateFlowAnalysis( network );
		
		try {
			sf.performEnd2EndAnalysis( f1 );
			System.out.println( "e2e SFA SC      : " + sf.getEnd2EndServiceCurve() );
			System.out.println( "     per server : " + sf.getServerLeftOverBetaMap().values().toString() );
			System.out.println( "delay bound     : " + sf.getDelayBound() );
			System.out.println( "backlog bound   : " + sf.getBacklogBound() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "SFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}

		Configuration.setArbMuxFlag( Flag.SERVERLOCAL_FLAG );
		
		assertEquals("SFA ARB delay", new Rational( 2855, 18 ), sf.getDelayBound() );
		assertEquals("SFA ARB backlog", new Rational( 7250, 9 ), sf.getBacklogBound() );
	}
	
	@Test
	public void f1_pmoo_arbMux()
	{
		System.out.println( "Flow of interest : " + f1.toString() );
		System.out.println();
		
		Configuration.setArbMuxFlag( Flag.GLOBALLY_ON );
		
		System.out.println( "--- PMOO Analysis (ARB_MUX) ---" );
		PmooSfaAnalysis pmoo = new PmooSfaAnalysis( network );
		
		try {
			pmoo.performEnd2EndAnalysis( f1 );
			System.out.println( "e2e PMOO SFA SC : " + pmoo.getEnd2EndServiceCurve() );
			System.out.println( "delay bound     : " + pmoo.getDelayBound() );
			System.out.println( "backlog bound   : " + pmoo.getBacklogBound() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "PMOO SFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}

		Configuration.setArbMuxFlag( Flag.SERVERLOCAL_FLAG );
		
		assertEquals("PMOO ARB delay", new Rational( 2095, 12 ), pmoo.getDelayBound() );
		assertEquals("PMOO ARB backlog", new Rational( 10625, 12 ), pmoo.getBacklogBound() );
	}
	
//--------------------Flow 2--------------------	
	@Test
	public void f2_tfa_fifoMux()
	{
		System.out.println( "Flow of interest : " + f2.toString() );
		System.out.println();
		
		Configuration.setArbMuxFlag( Flag.GLOBALLY_OFF );
		
		System.out.println( "--- Total Flow Analysis (FIFO_MUX) ---" );
		TotalFlowAnalysis tf = new TotalFlowAnalysis( network );
		
		try {
			tf.performEnd2EndAnalysis( f2 );
			System.out.println( "delay bound     : " + tf.getDelayBound() );
			System.out.println( "     per server : " + tf.getServerDelayBoundMap().values().toString() );
			System.out.println( "backlog bound   : " + tf.getBacklogBound() );
			System.out.println( "     per server : " + tf.getServerBacklogBoundMap().values().toString() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "TFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}

		Configuration.setArbMuxFlag( Flag.SERVERLOCAL_FLAG );
		
		assertEquals("TFA FIFO delay", new Rational( 1875, 32 ), tf.getDelayBound() );
		assertEquals("TFA FIFO backlog", new Rational( 3975, 8 ), tf.getBacklogBound() );
	}
	
	@Test
	public void f2_tfa_arbMux()
	{
		System.out.println( "Flow of interest : " + f2.toString() );
		System.out.println();
		
		Configuration.setArbMuxFlag( Flag.GLOBALLY_ON );
		
		System.out.println( "--- Total Flow Analysis (ARB_MUX) ---" );
		TotalFlowAnalysis tf = new TotalFlowAnalysis( network );
		
		try {
			tf.performEnd2EndAnalysis( f2 );
			System.out.println( "delay bound     : " + tf.getDelayBound() );
			System.out.println( "     per server : " + tf.getServerDelayBoundMap().values().toString() );
			System.out.println( "backlog bound   : " + tf.getBacklogBound() );
			System.out.println( "     per server : " + tf.getServerBacklogBoundMap().values().toString() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "TFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}

		Configuration.setArbMuxFlag( Flag.SERVERLOCAL_FLAG );
		
		assertEquals("TFA ARB delay", new Rational( 1105, 6 ), tf.getDelayBound() );
		assertEquals("TFA ARB backlog", new Rational( 2075, 3 ), tf.getBacklogBound() );
	}
	
	@Test
	public void f2_sfa_fifoMux()
	{
		System.out.println( "Flow of interest : " + f2.toString() );
		System.out.println();
		
		Configuration.setArbMuxFlag( Flag.GLOBALLY_OFF );
		
		System.out.println( "--- Separated Flow Analysis (FIFO_MUX) ---" );
		SeparateFlowAnalysis sf = new SeparateFlowAnalysis( network );

		try {
			sf.performEnd2EndAnalysis( f2 );
			System.out.println( "e2e SFA SC      : " + sf.getEnd2EndServiceCurve() );
			System.out.println( "     per server : " + sf.getServerLeftOverBetaMap().values().toString() );
			System.out.println( "delay bound     : " + sf.getDelayBound() );
			System.out.println( "backlog bound   : " + sf.getBacklogBound() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "SFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}

		Configuration.setArbMuxFlag( Flag.SERVERLOCAL_FLAG );

		assertEquals("SFA FIFO delay", new Rational( 3385, 64 ), sf.getDelayBound() );
		assertEquals("SFA FIFO backlog", new Rational( 17725, 64 ), sf.getBacklogBound() );
	}
	
	@Test
	public void f2_sfa_arbMux()
	{
		System.out.println( "Flow of interest : " + f2.toString() );
		System.out.println();
		
		Configuration.setArbMuxFlag( Flag.GLOBALLY_ON );
		
		System.out.println( "--- Separated Flow Analysis (ARB_MUX) ---" );
		SeparateFlowAnalysis sf = new SeparateFlowAnalysis( network );

		try {
			sf.performEnd2EndAnalysis( f2 );
			System.out.println( "e2e SFA SC      : " + sf.getEnd2EndServiceCurve() );
			System.out.println( "     per server : " + sf.getServerLeftOverBetaMap().values().toString() );
			System.out.println( "delay bound     : " + sf.getDelayBound() );
			System.out.println( "backlog bound   : " + sf.getBacklogBound() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "SFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}

		Configuration.setArbMuxFlag( Flag.SERVERLOCAL_FLAG );
		
		assertEquals("SFA ARB delay", new Rational( 4775, 54 ), sf.getDelayBound() );
		assertEquals("SFA ARB backlog", new Rational( 12275, 27 ), sf.getBacklogBound() );
	}
	
	@Test
	public void f2_pmoo_arbMux()
	{
		System.out.println( "Flow of interest : " + f2.toString() );
		System.out.println();

		Configuration.setArbMuxFlag( Flag.GLOBALLY_ON );
		
		System.out.println( "--- PMOO Analysis (ARB_MUX) ---" );
		PmooSfaAnalysis pmoo = new PmooSfaAnalysis( network );
		
		try {
			pmoo.performEnd2EndAnalysis( f2 );
			System.out.println( "e2e PMOO SFA SC : " + pmoo.getEnd2EndServiceCurve() );
			System.out.println( "delay bound     : " + pmoo.getDelayBound() );
			System.out.println( "backlog bound   : " + pmoo.getBacklogBound() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "PMOO SFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}

		Configuration.setArbMuxFlag( Flag.SERVERLOCAL_FLAG );
		
		assertEquals("PMOO ARB delay", new Rational( 305, 3 ), pmoo.getDelayBound() );
		assertEquals("PMOO ARB backlog", new Rational( 3125, 6 ), pmoo.getBacklogBound() );
	}
		
//--------------------Flow 3--------------------	
	@Test
	public void f3_tfa_fifoMux()
	{
		System.out.println( "Flow of interest : " + f3.toString() );
		System.out.println();
		
		Configuration.setArbMuxFlag( Flag.GLOBALLY_OFF );
		
		System.out.println( "--- Total Flow Analysis (FIFO_MUX) ---" );
		TotalFlowAnalysis tf = new TotalFlowAnalysis( network );
		
		try {
			tf.performEnd2EndAnalysis( f3 );
			System.out.println( "delay bound     : " + tf.getDelayBound() );
			System.out.println( "     per server : " + tf.getServerDelayBoundMap().values().toString() );
			System.out.println( "backlog bound   : " + tf.getBacklogBound() );
			System.out.println( "     per server : " + tf.getServerBacklogBoundMap().values().toString() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "TFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}

		Configuration.setArbMuxFlag( Flag.SERVERLOCAL_FLAG );
		
		assertEquals("TFA FIFO delay", new Rational( 845, 8 ), tf.getDelayBound() );
		assertEquals("TFA FIFO backlog", new Rational( 975 ), tf.getBacklogBound() );
	}
	
	@Test
	public void f3_tfa_arbMux()
	{
		System.out.println( "Flow of interest : " + f3.toString() );
		System.out.println();
		
		Configuration.setArbMuxFlag( Flag.GLOBALLY_ON );
		
		System.out.println( "--- Total Flow Analysis (ARB_MUX) ---" );
		TotalFlowAnalysis tf = new TotalFlowAnalysis( network );
		
		try {
			tf.performEnd2EndAnalysis( f3 );
			System.out.println( "delay bound     : " + tf.getDelayBound() );
			System.out.println( "     per server : " + tf.getServerDelayBoundMap().values().toString() );
			System.out.println( "backlog bound   : " + tf.getBacklogBound() );
			System.out.println( "     per server : " + tf.getServerBacklogBoundMap().values().toString() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "TFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}

		Configuration.setArbMuxFlag( Flag.SERVERLOCAL_FLAG );
		
		assertEquals("TFA ARB delay", new Rational( 462.5 ), tf.getDelayBound() );
		assertEquals("TFA ARB backlog", new Rational( 1400 ), tf.getBacklogBound() );
	}
	
	@Test
	public void f3_sfa_fifoMux()
	{
		System.out.println( "Flow of interest : " + f3.toString() );
		System.out.println();
		
		Configuration.setArbMuxFlag( Flag.GLOBALLY_OFF );
		
		System.out.println( "--- Separated Flow Analysis (FIFO_MUX) ---" );
		SeparateFlowAnalysis sf = new SeparateFlowAnalysis( network );

		try {
			sf.performEnd2EndAnalysis( f3 );
			System.out.println( "e2e SFA SC      : " + sf.getEnd2EndServiceCurve() );
			System.out.println( "     per server : " + sf.getServerLeftOverBetaMap().values().toString() );
			System.out.println( "delay bound     : " + sf.getDelayBound() );
			System.out.println( "backlog bound   : " + sf.getBacklogBound() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "SFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}

		Configuration.setArbMuxFlag( Flag.SERVERLOCAL_FLAG );
		
		assertEquals("SFA FIFO delay", new Rational( 5485, 64 ), sf.getDelayBound() );
		assertEquals("SFA FIFO backlog", new Rational( 28225, 64 ), sf.getBacklogBound() );
	}
	
	@Test
	public void f3_sfa_arbMux()
	{
		System.out.println( "Flow of interest : " + f3.toString() );
		System.out.println();
		
		Configuration.setArbMuxFlag( Flag.GLOBALLY_ON );
		
		System.out.println( "--- Separated Flow Analysis (ARB_MUX) ---" );
		SeparateFlowAnalysis sf = new SeparateFlowAnalysis( network );

		try {
			sf.performEnd2EndAnalysis( f3 );
			System.out.println( "e2e SFA SC      : " + sf.getEnd2EndServiceCurve() );
			System.out.println( "     per server : " + sf.getServerLeftOverBetaMap().values().toString() );
			System.out.println( "delay bound     : " + sf.getDelayBound() );
			System.out.println( "backlog bound   : " + sf.getBacklogBound() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "SFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}

		Configuration.setArbMuxFlag( Flag.SERVERLOCAL_FLAG );
		
		assertEquals("SFA ARB delay", new Rational( 1475, 9 ), sf.getDelayBound() );
		assertEquals("SFA ARB backlog", new Rational( 14975, 18 ), sf.getBacklogBound() );
	}
	
	@Test
	public void f3_pmoo_arbMux()
	{
		System.out.println( "Flow of interest : " + f3.toString() );
		System.out.println();
		
		Configuration.setArbMuxFlag( Flag.GLOBALLY_ON );
		
		System.out.println( "--- PMOO Analysis (ARB_MUX) ---" );
		PmooSfaAnalysis pmoo = new PmooSfaAnalysis( network );
		
		try {
			pmoo.performEnd2EndAnalysis( f3 );
			System.out.println( "e2e PMOO SFA SC : " + pmoo.getEnd2EndServiceCurve() );
			System.out.println( "delay bound     : " + pmoo.getDelayBound() );
			System.out.println( "backlog bound   : " + pmoo.getBacklogBound() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "PMOO SFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}

		Configuration.setArbMuxFlag( Flag.SERVERLOCAL_FLAG );
		
		assertEquals("PMOO ARB delay", new Rational( 1145, 6 ), pmoo.getDelayBound() );
		assertEquals("PMOO ARB backlog", new Rational( 2900, 3 ), pmoo.getBacklogBound() );
	}
}