package unikl.disco.tests;

import org.junit.BeforeClass;
import org.junit.Test;
import static org.junit.Assert.assertEquals;

import unikl.disco.curves.Curve;

import unikl.disco.misc.Rational;
import unikl.disco.nc.TotalFlowAnalysis;
import unikl.disco.nc.SeparateFlowAnalysis;
import unikl.disco.nc.PmooSfaAnalysis;

import unikl.disco.network.Flow;
import unikl.disco.network.Network;
import unikl.disco.network.Server;

public class Node_1Flow
{
	static final Curve service_curve = Curve.createRateLatency( 10, 10 );
	static final Curve arrival_curve = Curve.createTokenBucket( 5, 25 );
	static final Curve max_service_curve = Curve.createRateLatency( 15, 5 );
	
	static Network network;
	static Server s0;
	static Flow f0;

	@BeforeClass
	static public void createNetwork()
	{
		network = new Network();
		s0 = network.addServer( service_curve, max_service_curve );
		s0.setUseGamma( false );
		s0.setUseExtraGamma( false );
	
		try {	
			f0 = network.addFlow( arrival_curve, s0 );
		} catch (Exception e) {
			System.out.println( e.toString() );
			return;
		}
		
		System.out.println( "Flow of interest : " + f0.toString() );
		System.out.println();
	}
	
	@Test
	public void f0_tfa_fifoMux()
	{
		s0.setUseFifoMultiplexing( true ); // default = false
		System.out.println("--- Total Flow Analysis (FIFO_MUX)---");
		TotalFlowAnalysis tf = new TotalFlowAnalysis( network );
				
		try {
			tf.performEnd2EndAnalysis( f0 );
			System.out.println( "delay bound     : " + tf.getDelayBound() );
			System.out.println( "     per server : " + tf.getServerDelayBoundMap().values().toString() );
			System.out.println( "backlog bound   : " + tf.getBacklogBound() );
			System.out.println( "     per server : " + tf.getServerBacklogBoundMap().values().toString() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "TFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}
		
		assertEquals("TFA FIFO delay", new Rational( 12.5 ), tf.getDelayBound() );
		assertEquals("TFA FIFO backlog", new Rational( 75 ), tf.getBacklogBound() );
	}
	
	@Test
	public void f0_tfa_arbMux()
	{
		s0.setUseArbitraryMultiplexing( true ); // s0.useArbitraryMultiplexing() != s0.useFifoMultiplexing()
		System.out.println("--- Total Flow Analysis (ARB_MUX)---");
		TotalFlowAnalysis tf = new TotalFlowAnalysis( network );
				
		try {
			tf.performEnd2EndAnalysis( f0 );
			System.out.println( "delay bound     : " + tf.getDelayBound() );
			System.out.println( "     per server : " + tf.getServerDelayBoundMap().values().toString() );
			System.out.println( "backlog bound   : " + tf.getBacklogBound() );
			System.out.println( "     per server : " + tf.getServerBacklogBoundMap().values().toString() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "TFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}
		
		assertEquals("TFA ARB delay", new Rational( 12.5 ), tf.getDelayBound() );
		assertEquals("TFA ARB backlog", new Rational( 75 ), tf.getBacklogBound() );
	}
	
	@Test
	public void f0_sfa_fifoMux()
	{
		s0.setUseFifoMultiplexing( true );
		System.out.println( "--- Separated Flow Analysis (FIFO_MUX)---" );
		SeparateFlowAnalysis sf = new SeparateFlowAnalysis( network );

		try {
			sf.performEnd2EndAnalysis( f0 );
			System.out.println( "e2e SFA SC      : " + sf.getEnd2EndServiceCurve() );
			System.out.println( "     per server : " + sf.getServerLeftOverBetaMap().values().toString() );
			System.out.println( "delay bound     : " + sf.getDelayBound() );
			System.out.println( "backlog bound   : " + sf.getBacklogBound() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "SFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}
		
		assertEquals("SFA FIFO delay", new Rational( 12.5 ), sf.getDelayBound() );
		assertEquals("SFA FIFO backlog", new Rational( 75 ), sf.getBacklogBound() );
	}
	
	@Test
	public void f0_sfa_arbMux()
	{
		s0.setUseArbitraryMultiplexing( true );
		System.out.println( "--- Separated Flow Analysis (ARB_MUX)---" );
		SeparateFlowAnalysis sf = new SeparateFlowAnalysis( network );

		try {
			sf.performEnd2EndAnalysis( f0 );
			System.out.println( "e2e SFA SC      : " + sf.getEnd2EndServiceCurve() );
			System.out.println( "     per server : " + sf.getServerLeftOverBetaMap().values().toString() );
			System.out.println( "delay bound     : " + sf.getDelayBound() );
			System.out.println( "backlog bound   : " + sf.getBacklogBound() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "SFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}
		
		assertEquals("SFA ARB delay", new Rational( 12.5 ), sf.getDelayBound() );
		assertEquals("SFA ARB backlog", new Rational( 75 ), sf.getBacklogBound() );
	}

	@Test
	public void f0_pmoo_arbMux()
	{
		s0.setUseArbitraryMultiplexing( true );
		System.out.println( "--- PMOO Analysis (ARB_MUX) ---" );
		PmooSfaAnalysis pmoo = new PmooSfaAnalysis( network );
		
		try {
			pmoo.performEnd2EndAnalysis( f0 );
			System.out.println( "e2e PMOO SFA SC : " + pmoo.getEnd2EndServiceCurve() );
			System.out.println( "delay bound     : " + pmoo.getDelayBound() );
			System.out.println( "backlog bound   : " + pmoo.getBacklogBound() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "PMOO SFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}

		assertEquals("PMOO ARB delay", new Rational( 12.5 ), pmoo.getDelayBound() );
		assertEquals("PMOO ARB backlog", new Rational( 75 ), pmoo.getBacklogBound() );
	}
}
