package unikl.disco.tests;

import static org.junit.Assert.assertEquals;
import org.junit.BeforeClass;
import org.junit.Test;

import unikl.disco.curves.Curve;
import unikl.disco.misc.Rational;
import unikl.disco.nc.Configuration;
import unikl.disco.nc.TotalFlowAnalysis;
import unikl.disco.nc.SeparateFlowAnalysis;
import unikl.disco.nc.PmooSfaAnalysis;
import unikl.disco.nc.Configuration.Flag;

import unikl.disco.network.Flow;
import unikl.disco.network.Network;
import unikl.disco.network.Server;

public class Tree_1SC_3Flows_1AC_3Paths
{
	static final Curve service_curve = Curve.createRateLatency( 20, 20 );
	static final Curve arrival_curve = Curve.createTokenBucket( 5, 25 );
	
	static Network network;
	static Server s0, s1, s2, s3, s4, s5, s6;
	static Flow f0, f1, f2;
	
	@BeforeClass
	static public void createNetwork()
	{
		network = new Network();
		s0 = network.addServer( service_curve );
		s1 = network.addServer( service_curve );
		s2 = network.addServer( service_curve );
		s3 = network.addServer( service_curve );
		s4 = network.addServer( service_curve );
		s5 = network.addServer( service_curve );
		s6 = network.addServer( service_curve );

		try {
			network.addLink( s0, s1 );
			network.addLink( s1, s2 );
			network.addLink( s2, s5 );
			network.addLink( s3, s4 );
			network.addLink( s4, s5 );
			network.addLink( s5, s6 );
		} catch (Exception e) {
			System.out.println( e.toString() );
			return;
		}

		try {	
			f0 = network.addFlow( arrival_curve, s1, s6 );
			f1 = network.addFlow( arrival_curve, s0, s6 );
			f2 = network.addFlow( arrival_curve, s3, s6 );
		} catch (Exception e) {
			System.out.println( e.toString() );
			return;
		}
	}

//--------------------Flow 0--------------------	
	@Test
	public void f0_tfa_fifoMux()
	{
		System.out.println( "Flow of interest : " + f0.toString() );
		System.out.println();
		
		Configuration.setArbMuxFlag( Flag.GLOBALLY_OFF );
		
		System.out.println( "--- Total Flow Analysis (FIFO_MUX) ---" );
		TotalFlowAnalysis tf = new TotalFlowAnalysis( network );
		
		try {
			tf.performEnd2EndAnalysis( f0 );
			System.out.println( "delay bound     : " + tf.getDelayBound() );
			System.out.println( "     per server : " + tf.getServerDelayBoundMap().values().toString() );
			System.out.println( "backlog bound   : " + tf.getBacklogBound() );
			System.out.println( "     per server : " + tf.getServerBacklogBoundMap().values().toString() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "TFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}

		Configuration.setArbMuxFlag( Flag.SERVERLOCAL_FLAG );
		
		assertEquals("TFA FIFO delay", new Rational( 395, 2 ), tf.getDelayBound() );
		assertEquals("TFA FIFO backlog", new Rational( 1375 ), tf.getBacklogBound() );
	}
	
	@Test
	public void f0_tfa_arbMux()
	{
		System.out.println( "Flow of interest : " + f0.toString() );
		System.out.println();
		
		Configuration.setArbMuxFlag( Flag.GLOBALLY_ON );
		
		System.out.println( "--- Total Flow Analysis (ARB_MUX) ---" );
		TotalFlowAnalysis tf = new TotalFlowAnalysis( network );
		
		try {
			tf.performEnd2EndAnalysis( f0 );
			System.out.println( "delay bound     : " + tf.getDelayBound() );
			System.out.println( "     per server : " + tf.getServerDelayBoundMap().values().toString() );
			System.out.println( "backlog bound   : " + tf.getBacklogBound() );
			System.out.println( "     per server : " + tf.getServerBacklogBoundMap().values().toString() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "TFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}

		Configuration.setArbMuxFlag( Flag.SERVERLOCAL_FLAG );

		assertEquals("TFA ARB delay", new Rational( 660 ), tf.getDelayBound() );
		assertEquals("TFA ARB backlog", new Rational( 1375 ), tf.getBacklogBound() );
	}

	@Test
	public void f0_sfa_fifoMux()
	{
		System.out.println( "Flow of interest : " + f0.toString() );
		System.out.println();

		Configuration.setArbMuxFlag( Flag.GLOBALLY_OFF );
		
		System.out.println( "--- Separated Flow Analysis (FIFO_MUX) ---" );
		SeparateFlowAnalysis sf = new SeparateFlowAnalysis( network );

		try {
			sf.performEnd2EndAnalysis( f0 );
			System.out.println( "e2e SFA SC      : " + sf.getEnd2EndServiceCurve() );
			System.out.println( "     per server : " + sf.getServerLeftOverBetaMap().values().toString() );
			System.out.println( "delay bound     : " + sf.getDelayBound() );
			System.out.println( "backlog bound   : " + sf.getBacklogBound() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "SFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}

		Configuration.setArbMuxFlag( Flag.SERVERLOCAL_FLAG );
		
		assertEquals("SFA FIFO delay", new Rational( 165 ), sf.getDelayBound() );
		assertEquals("SFA FIFO backlog", new Rational( 1675, 2 ), sf.getBacklogBound() );
	}

	@Test
	public void f0_sfa_arbMux()
	{
		System.out.println( "Flow of interest : " + f0.toString() );
		System.out.println();
		
		Configuration.setArbMuxFlag( Flag.GLOBALLY_ON );
		
		System.out.println( "--- Separated Flow Analysis (ARB_MUX) ---" );
		SeparateFlowAnalysis sf = new SeparateFlowAnalysis( network );

		try {
			sf.performEnd2EndAnalysis( f0 );
			System.out.println( "e2e SFA SC      : " + sf.getEnd2EndServiceCurve() );
			System.out.println( "     per server : " + sf.getServerLeftOverBetaMap().values().toString() );
			System.out.println( "delay bound     : " + sf.getDelayBound() );
			System.out.println( "backlog bound   : " + sf.getBacklogBound() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "SFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}

		Configuration.setArbMuxFlag( Flag.SERVERLOCAL_FLAG );
		
		assertEquals("SFA ARB delay", new Rational( 1735, 6 ), sf.getDelayBound() );
		assertEquals("SFA ARB backlog", new Rational( 4375, 3 ), sf.getBacklogBound() );
	}
	
	@Test
	public void f0_pmoo_arbMux()
	{
		System.out.println( "Flow of interest : " + f0.toString() );
		System.out.println();
		
		Configuration.setArbMuxFlag( Flag.GLOBALLY_ON );
		
		System.out.println( "--- PMOO Analysis (ARB_MUX) ---" );
		PmooSfaAnalysis pmoo = new PmooSfaAnalysis( network );
		
		try {
			pmoo.performEnd2EndAnalysis( f0 );
			System.out.println( "e2e PMOO SFA SC : " + pmoo.getEnd2EndServiceCurve() );
			System.out.println( "delay bound     : " + pmoo.getDelayBound() );
			System.out.println( "backlog bound   : " + pmoo.getBacklogBound() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "PMOO SFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}

		Configuration.setArbMuxFlag( Flag.SERVERLOCAL_FLAG );

		assertEquals("PMOO ARB delay", new Rational( 355, 2 ), pmoo.getDelayBound() );
		assertEquals("PMOO ARB backlog", new Rational( 900 ), pmoo.getBacklogBound() );
	}

//--------------------Flow 1--------------------	
	@Test
	public void f1_tfa_fifoMux()
	{
		System.out.println( "Flow of interest : " + f1.toString() );
		System.out.println();
		
		Configuration.setArbMuxFlag( Flag.GLOBALLY_OFF );
		
		System.out.println( "--- Total Flow Analysis (FIFO_MUX) ---" );
		TotalFlowAnalysis tf = new TotalFlowAnalysis( network );
		
		try {
			tf.performEnd2EndAnalysis( f1 );
			System.out.println( "delay bound     : " + tf.getDelayBound() );
			System.out.println( "     per server : " + tf.getServerDelayBoundMap().values().toString() );
			System.out.println( "backlog bound   : " + tf.getBacklogBound() );
			System.out.println( "     per server : " + tf.getServerBacklogBoundMap().values().toString() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "TFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}

		Configuration.setArbMuxFlag( Flag.SERVERLOCAL_FLAG );
		
		assertEquals("TFA FIFO delay", new Rational( 875, 4 ), tf.getDelayBound() );
		assertEquals("TFA FIFO backlog", new Rational( 1375 ), tf.getBacklogBound() );
	}
	
	@Test
	public void f1_tfa_arbMux()
	{
		System.out.println( "Flow of interest : " + f1.toString() );
		System.out.println();
		
		Configuration.setArbMuxFlag( Flag.GLOBALLY_ON );
		
		System.out.println( "--- Total Flow Analysis (ARB_MUX) ---" );
		TotalFlowAnalysis tf = new TotalFlowAnalysis( network );
		
		try {
			tf.performEnd2EndAnalysis( f1 );
			System.out.println( "delay bound     : " + tf.getDelayBound() );
			System.out.println( "     per server : " + tf.getServerDelayBoundMap().values().toString() );
			System.out.println( "backlog bound   : " + tf.getBacklogBound() );
			System.out.println( "     per server : " + tf.getServerBacklogBoundMap().values().toString() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "TFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}

		Configuration.setArbMuxFlag( Flag.SERVERLOCAL_FLAG );
		
		assertEquals("TFA ARB delay", new Rational( 2725, 4 ), tf.getDelayBound() );
		assertEquals("TFA ARB backlog", new Rational( 1375 ), tf.getBacklogBound() );
	}
	
	@Test
	public void f1_sfa_fifoMux()
	{
		System.out.println( "Flow of interest : " + f1.toString() );
		System.out.println();
		
		Configuration.setArbMuxFlag( Flag.GLOBALLY_OFF );
		
		System.out.println( "--- Separated Flow Analysis (FIFO_MUX) ---" );
		SeparateFlowAnalysis sf = new SeparateFlowAnalysis( network );

		try {
			sf.performEnd2EndAnalysis( f1 );
			System.out.println( "e2e SFA SC      : " + sf.getEnd2EndServiceCurve() );
			System.out.println( "     per server : " + sf.getServerLeftOverBetaMap().values().toString() );
			System.out.println( "delay bound     : " + sf.getDelayBound() );
			System.out.println( "backlog bound   : " + sf.getBacklogBound() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "SFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}

		Configuration.setArbMuxFlag( Flag.SERVERLOCAL_FLAG );
		
		assertEquals("SFA FIFO delay", new Rational( 165 ), sf.getDelayBound() );
		assertEquals("SFA FIFO backlog", new Rational( 1675, 2 ), sf.getBacklogBound() );
	}
	
	@Test
	public void f1_sfa_arbMux()
	{
		System.out.println( "Flow of interest : " + f1.toString() );
		System.out.println();
		
		Configuration.setArbMuxFlag( Flag.GLOBALLY_ON );
		
		System.out.println( "--- Separated Flow Analysis (ARB_MUX) ---" );
		SeparateFlowAnalysis sf = new SeparateFlowAnalysis( network );
		
		try {
			sf.performEnd2EndAnalysis( f1 );
			System.out.println( "e2e SFA SC      : " + sf.getEnd2EndServiceCurve() );
			System.out.println( "     per server : " + sf.getServerLeftOverBetaMap().values().toString() );
			System.out.println( "delay bound     : " + sf.getDelayBound() );
			System.out.println( "backlog bound   : " + sf.getBacklogBound() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "SFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}

		Configuration.setArbMuxFlag( Flag.SERVERLOCAL_FLAG );
		
		assertEquals("SFA ARB delay", new Rational( 1655, 6 ), sf.getDelayBound() );
		assertEquals("SFA ARB backlog", new Rational( 4175, 3 ), sf.getBacklogBound() );
	}
	
	@Test
	public void f1_pmoo_arbMux()
	{
		System.out.println( "Flow of interest : " + f1.toString() );
		System.out.println();
		
		Configuration.setArbMuxFlag( Flag.GLOBALLY_ON );
		
		System.out.println( "--- PMOO Analysis (ARB_MUX) ---" );
		PmooSfaAnalysis pmoo = new PmooSfaAnalysis( network );
		
		try {
			pmoo.performEnd2EndAnalysis( f1 );
			System.out.println( "e2e PMOO SFA SC : " + pmoo.getEnd2EndServiceCurve() );
			System.out.println( "delay bound     : " + pmoo.getDelayBound() );
			System.out.println( "backlog bound   : " + pmoo.getBacklogBound() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "PMOO SFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}

		Configuration.setArbMuxFlag( Flag.SERVERLOCAL_FLAG );
		
		assertEquals("PMOO ARB delay", new Rational( 375, 2  ), pmoo.getDelayBound() );
		assertEquals("PMOO ARB backlog", new Rational( 950 ), pmoo.getBacklogBound() );
	}
	
//--------------------Flow 2--------------------	
	@Test
	public void f2_tfa_fifoMux()
	{
		System.out.println( "Flow of interest : " + f2.toString() );
		System.out.println();
		
		Configuration.setArbMuxFlag( Flag.GLOBALLY_OFF );
		
		System.out.println( "--- Total Flow Analysis (FIFO_MUX) ---" );
		TotalFlowAnalysis tf = new TotalFlowAnalysis( network );
		
		try {
			tf.performEnd2EndAnalysis( f2 );
			System.out.println( "delay bound     : " + tf.getDelayBound() );
			System.out.println( "     per server : " + tf.getServerDelayBoundMap().values().toString() );
			System.out.println( "backlog bound   : " + tf.getBacklogBound() );
			System.out.println( "     per server : " + tf.getServerBacklogBoundMap().values().toString() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "TFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}

		Configuration.setArbMuxFlag( Flag.SERVERLOCAL_FLAG );
		
		assertEquals("TFA FIFO delay", new Rational( 180 ), tf.getDelayBound() );
		assertEquals("TFA FIFO backlog", new Rational( 1375 ), tf.getBacklogBound() );
	}
	
	@Test
	public void f2_tfa_arbMux()
	{
		System.out.println( "Flow of interest : " + f2.toString() );
		System.out.println();
		
		Configuration.setArbMuxFlag( Flag.GLOBALLY_ON );
		
		System.out.println( "--- Total Flow Analysis (ARB_MUX) ---" );
		TotalFlowAnalysis tf = new TotalFlowAnalysis( network );
		
		try {
			tf.performEnd2EndAnalysis( f2 );
			System.out.println( "delay bound     : " + tf.getDelayBound() );
			System.out.println( "     per server : " + tf.getServerDelayBoundMap().values().toString() );
			System.out.println( "backlog bound   : " + tf.getBacklogBound() );
			System.out.println( "     per server : " + tf.getServerBacklogBoundMap().values().toString() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "TFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}

		Configuration.setArbMuxFlag( Flag.SERVERLOCAL_FLAG );
		
		assertEquals("TFA ARB delay", new Rational( 1155, 2 ), tf.getDelayBound() );
		assertEquals("TFA ARB backlog", new Rational( 1375 ), tf.getBacklogBound() );
	}
	
	@Test
	public void f2_sfa_fifoMux()
	{
		System.out.println( "Flow of interest : " + f2.toString() );
		System.out.println();
		
		Configuration.setArbMuxFlag( Flag.GLOBALLY_OFF );
		
		System.out.println( "--- Separated Flow Analysis (FIFO_MUX) ---" );
		SeparateFlowAnalysis sf = new SeparateFlowAnalysis( network );

		try {
			sf.performEnd2EndAnalysis( f2 );
			System.out.println( "e2e SFA SC      : " + sf.getEnd2EndServiceCurve() );
			System.out.println( "     per server : " + sf.getServerLeftOverBetaMap().values().toString() );
			System.out.println( "delay bound     : " + sf.getDelayBound() );
			System.out.println( "backlog bound   : " + sf.getBacklogBound() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "SFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}

		Configuration.setArbMuxFlag( Flag.SERVERLOCAL_FLAG );
		
		assertEquals("SFA FIFO delay", new Rational( 295, 2 ), sf.getDelayBound() );
		assertEquals("SFA FIFO backlog", new Rational( 750 ), sf.getBacklogBound() );
	}
	
	@Test
	public void f2_sfa_arbMux()
	{
		System.out.println( "Flow of interest : " + f2.toString() );
		System.out.println();
		
		Configuration.setArbMuxFlag( Flag.GLOBALLY_ON );
		
		System.out.println( "--- Separated Flow Analysis (ARB_MUX) ---" );
		SeparateFlowAnalysis sf = new SeparateFlowAnalysis( network );

		try {
			sf.performEnd2EndAnalysis( f2 );
			System.out.println( "e2e SFA SC      : " + sf.getEnd2EndServiceCurve() );
			System.out.println( "     per server : " + sf.getServerLeftOverBetaMap().values().toString() );
			System.out.println( "delay bound     : " + sf.getDelayBound() );
			System.out.println( "backlog bound   : " + sf.getBacklogBound() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "SFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}

		Configuration.setArbMuxFlag( Flag.SERVERLOCAL_FLAG );
		
		assertEquals("SFA ARB delay", new Rational( 505, 2 ), sf.getDelayBound() );
		assertEquals("SFA ARB backlog", new Rational( 1275 ), sf.getBacklogBound() );
	}
	
	@Test
	public void f2_pmoo_arbMux()
	{
		System.out.println( "Flow of interest : " + f2.toString() );
		System.out.println();

		Configuration.setArbMuxFlag( Flag.GLOBALLY_ON );
		
		System.out.println( "--- PMOO Analysis (ARB_MUX) ---" );
		PmooSfaAnalysis pmoo = new PmooSfaAnalysis( network );
		
		try {
			pmoo.performEnd2EndAnalysis( f2 );
			System.out.println( "e2e PMOO SFA SC : " + pmoo.getEnd2EndServiceCurve() );
			System.out.println( "delay bound     : " + pmoo.getDelayBound() );
			System.out.println( "backlog bound   : " + pmoo.getBacklogBound() );
			System.out.println();
		} catch (Exception e) {
			System.out.println( "PMOO SFA analysis failed" );
			System.out.println( e.toString() );
			System.out.println();
		}

		Configuration.setArbMuxFlag( Flag.SERVERLOCAL_FLAG );
		
		assertEquals("PMOO ARB delay", new Rational( 355, 2 ), pmoo.getDelayBound() );
		assertEquals("PMOO ARB backlog", new Rational( 900 ), pmoo.getBacklogBound() );
	}
}